@@ -13,7 +13,7 @@ use vars qw(
 ## Inheritance and Versioning ##
 
 @ISA     = qw( Exporter );
-$VERSION = '0.07';
+$VERSION = '0.15';
 
 ## General Definitions ##
 
@@ -521,6 +521,27 @@ sub _handle_event
             ##
             $this->parent->refresh_callback(undef);
         }
+
+        # trigger post-transition actions
+        $this->_trigger_post_transition_action($state, $next_state);
+    }
+}
+
+sub _trigger_post_transition_action
+{
+    my ($this, $pre_state, $pos_state) = @_;
+
+    # TODO:
+    #
+    # This needs to be broken out into a separate table similar to $BGP_FSM
+    # which triggers actions prior to state transition. Or, alternately,
+    # $BGP_FSM could be augmented to have an array of subrefs, one each for the
+    # pre- and post- transition action, rather than the current scalar subref.
+    # But I'm too lazy to refactor the entire table right now, so just handle
+    # the single current use case of firing the ESTABLISHED callback...
+
+    if (($pre_state == BGP_STATE_OPEN_CONFIRM) && ($pos_state == BGP_STATE_ESTABLISHED)) {
+        $this->parent->established_callback();
     }
 }
 
@@ -714,6 +735,7 @@ sub _close_session
     $this->{_hold_timer} = undef;
     $this->{_keep_alive_timer} = undef;
     $this->{_connect_retry_timer} = undef;
+    $this->{_message_queue} = [];
 
     return ( BGP_STATE_IDLE );
 }
@@ -809,6 +831,7 @@ sub _handle_receive_notification_message
     my $error;
 
     $error = $this->_decode_bgp_notification_message($this->_dequeue_message());
+
     $this->_close_session();
 
     # invoke user callback function
@@ -1122,7 +1145,20 @@ sub _encode_bgp_open_message
 
     # encode optional parameters and length (only refresh supported)
     my $opt = '';
-    $opt .= pack('cc',2,0) if $this->parent->this_can_refresh;
+
+    if ($this->parent->support_capabilities) {
+        # RFC5492
+        # Format is <2> <capability_len> <cap_code> <data_len>
+
+        # Parameter type 2, length 6, capability 1 with a length of 4
+        # Address family 1 (IPv4), reserved bit 0, type 1 (unicast)
+        $opt .= pack('ccccncc',2,6,1,4,1,0,1) if $this->parent->support_mbgp;
+
+        # Both the standard (2) and Cisco (128) capabilities are sent
+        $opt .= pack('cccc',2,2,2,0) if $this->parent->this_can_refresh;
+        $opt .= pack('cccc',2,2,128,0) if $this->parent->this_can_refresh;
+    }
+
     $buffer = pack('C', length($opt)) . $opt;
 
     # encode BGP Identifier field
